import { loadScript, getLSP, sanitizeInlineStyle } from "@bhsd/common";
import { styleLint } from "@bhsd/common/dist/stylelint";
const offsetAt = (range, line, column) => {
  if (line === -2) {
    return range[0];
  }
  return line === 0 ? range[1] : range[0] + column;
};
const getWikiLinter = async (opt, obj) => {
  const DIR = "npm/wikiparser-node/extensions/dist", lang = opt == null ? void 0 : opt["i18n"];
  await loadScript(`${DIR}/base.min.js`, "wikiparse");
  await loadScript(`${DIR}/lsp.min.js`, "wikiparse.LanguageService");
  if (typeof lang === "string") {
    try {
      const i18n = await (await fetch(`${wikiparse.CDN}/i18n/${lang.toLowerCase()}.json`)).json();
      wikiparse.setI18N(i18n);
    } catch {
    }
  }
  const lsp = getLSP(obj, opt == null ? void 0 : opt["include"]);
  return async (text, config) => {
    const diagnostics = (await lsp.provideDiagnostics(text)).filter(
      ({ code, severity }) => {
        var _a;
        return Number((_a = config == null ? void 0 : config[code]) != null ? _a : 2) > Number(severity === 2);
      }
    ), tokens = "findStyleTokens" in lsp && (config == null ? void 0 : config["invalid-css"]) !== "0" ? await lsp.findStyleTokens() : [];
    if (tokens.length === 0) {
      return diagnostics;
    }
    const cssLint = await getCssLinter();
    return [
      ...diagnostics,
      ...(await cssLint(
        tokens.map(({ childNodes, type, tag }, i) => `${type === "ext-attr" ? "div" : tag}#${i}{
${sanitizeInlineStyle(childNodes[1].childNodes[0].data).replace(/\n/gu, " ")}
}`).join("\n")
      )).map(({ line, column, endLine, endColumn, rule, severity, text: message }) => {
        const i = Math.ceil(line / 3), { range } = tokens[i - 1].childNodes[1].childNodes[0], from = offsetAt(range, line - 3 * i, column - 1);
        return {
          from,
          to: endLine === void 0 ? from : offsetAt(range, endLine - 3 * i, endColumn - 1),
          severity: severity === "error" ? 1 : 2,
          source: "Stylelint",
          code: rule,
          message
        };
      })
    ];
  };
};
const getJsLinter = async () => {
  var _a;
  await loadScript("npm/eslint-linter-browserify@8.57.0/linter.min.js", "eslint", true);
  const esLinter = new eslint.Linter(), conf = {
    env: { browser: true, es2024: true },
    parserOptions: { ecmaVersion: 15, sourceType: "module" },
    rules: {}
  };
  for (const [name, { meta }] of esLinter.getRules()) {
    if ((_a = meta == null ? void 0 : meta.docs) == null ? void 0 : _a.recommended) {
      conf.rules[name] = 2;
    }
  }
  return (text, opt) => esLinter.verify(text, { ...conf, ...opt });
};
const getCssLinter = async () => {
  await loadScript("npm/stylelint-bundle", "stylelint");
  return (code, opt) => styleLint(stylelint, code, opt == null ? void 0 : opt["rules"]);
};
const getLuaLinter = async () => {
  await loadScript("npm/luacheck-browserify/dist/index.min.js", "luacheck");
  const luachecker = await luacheck(void 0);
  return async (text) => (await luachecker.queue(text)).filter(({ severity }) => severity);
};
const getJsonLinter = () => (str) => {
  var _a, _b, _c;
  try {
    if (str.trim()) {
      JSON.parse(str);
    }
  } catch (e) {
    if (e instanceof SyntaxError) {
      const { message } = e, line = (_a = /\bline (\d+)/u.exec(message)) == null ? void 0 : _a[1], column = (_b = /\bcolumn (\d+)/u.exec(message)) == null ? void 0 : _b[1], position = (_c = /\bposition (\d+)/u.exec(message)) == null ? void 0 : _c[1];
      return [
        {
          message,
          severity: "error",
          line,
          column,
          position
        }
      ];
    }
  }
  return [];
};
export {
  getCssLinter,
  getJsLinter,
  getJsonLinter,
  getLuaLinter,
  getWikiLinter
};
